#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>
#include <dirent.h>
#include <fcntl.h>
#include <inttypes.h>

#include <sys/mman.h>
#include <sys/types.h>

#include <xenctrl.h>

#include "xennerctrl.h"
#include "xenner.h"

/* -------------------------------------------------------------- */

static int do_stats(int domid)
{
    uint64_t           last_faults[XEN_FAULT_MAX];
    uint64_t           last_hcalls[XEN_HCALL_MAX];
    struct xennerdom   *dom;
    struct stat        st;

    dom = xenner_get_dom(domid);
    if (NULL == dom) {
	fprintf(stderr, "failed to open vmcore file\n");
	goto cleanup;
    }
    if (!dom->vminfo) {
	fprintf(stderr, "failed to parse vmcore file\n");
	goto cleanup;
    }

    fprintf(stderr, "collecting stats ...\n");
    for (;;) {
	memcpy(last_hcalls, dom->vminfo->hcalls, sizeof(last_hcalls));
	memcpy(last_faults, dom->vminfo->faults, sizeof(last_faults));
	sleep(1);
	if (dom->vminfo->dying) {
	    fprintf(stderr, "\nvm dying\n");
	    break;
	}
	if (-1 == stat(dom->filename, &st)) {
	    fprintf(stderr, "\nvm gone\n");
	    break;
	}
	fprintf(stderr,
		"\x1b[H" /* home */
		"\x1b[J" /* cls  */
		"<%s>\n", dom->filename);
	print_hcall_stats(NULL, dom->vminfo->hcalls, last_hcalls);
	print_fault_stats(NULL, dom->vminfo->faults, last_faults);
    }

cleanup:
    if (dom)
	xenner_put_dom(dom);
    return 0;
}

int main(int argc, char *argv[])
{
    xc_dominfo_t info;
    int xc;

    if (argc == 2)
	return do_stats(atoi(argv[1]));

    xc = xc_interface_open();
    for (;;) {
	if (1 == xc_domain_getinfo(xc, 1, 1, &info)) {
	    do_stats(info.domid);
	    sleep(1);
	} else {
	    fprintf(stderr, "no vm running\n");
	    sleep(5);
	}
    }
    xc_interface_close(xc);
}
